from Utility import *
from Config import *
from Process import *
import sys

# GetCodeFirstMigration();
# sys.argv[1]

ReceivedModuleId = "27797F0C-5BAF-42E2-8D50-FD637B6CCE17"; #"4A37B20E-97E3-4BF9-AD5D-73CEC6068857";
Platform = 0; #0;
BuildEnv = 1; #0;
ProcessParam(ReceivedModuleId, Platform, BuildEnv)

# if __name__ == "__main__":
#     if sys.argv:
#         if len(sys.argv) == 3 or len(sys.argv) == 4:
#             ReceivedModuleId = sys.argv[1]; #"4A37B20E-97E3-4BF9-AD5D-73CEC6068857";
#             Platform = sys.argv[2]; #0;
#             print("Received Module Id = " + ReceivedModuleId)
#             print("Platform = " + Platform)
#             if(len(sys.argv) == 4):
#                 IsLocalCompile = sys.argv[3];
#                 print("Is Local Compile Flag = " + IsLocalCompile)
#                 ProcessParam(ReceivedModuleId, Platform, IsLocalCompile);
#             else:
#                 ProcessParam(ReceivedModuleId, Platform);
#         elif len(sys.argv) == 1:
#             print("No Argument");
#         else:
#             print(len(sys.argv))
#             print("Argument length not equal to 3");

# ProcessParam("4F536062-7087-474B-AEB3-15872AF0858C", 0, 1);

# CurrentModule = GetCurrentModule(ReceivedModuleId);
# print(CurrentModule)

# IsDirectoryExist = None;
# if CurrentModule:
#     IsDirectoryExist = CheckDirectoryExist(CurrentModule["ProjectLocalPath"]);

# if(IsDirectoryExist):
#     StartProcess(CurrentModule, Platform);
# else:
#     EndProgram("Directory Not Exist End Program");

# print("End")

# CurrentModule = GetCurrentModule("4A37B20E-97E3-4BF9-AD5D-73CEC6068857");

